#!/usr/bin/perl
##############################################################################
# Owner : Martin Rieder (martin.rieder@schwindt.eu)                          #
#                                                                            #
# Name               : sctrep.pl                                             #
# Intention          : Replication from CATREP_SERVER to clients             #
# Require            : perl                                                  #
#                                                                            #
# v1.00 23.08.12 mrd : created                                               #
# v2.00 07.10.14 mrd : scan-mode added                                       #
# v2.01 21.10.14 mrd : Bugfix (fdec\site\all not replicated)                 #
# v2.02 27.10.14 mrd : Bugfix in Delete-Subroutine                           #
# v2.03 03.11.14 mrd : Fix in Source-Routine (Multi-CatInstallPath)          #
# v2.04 20.04.15 mrd : More robust call of robocopy.exe						 #
# v3.00 17.08.16 mrd : Added error reporting and other enhancements          #
# v3.01 23.09.16 mrd : Create Logstore if not exist                          #
# v3.02 14.12.16 mrd : More robust about spaces in pathnames                 #
# v3.03 11.12.19 lnt : Scan-mode supports Whitespaces and Tabs in			 # 
#					   *InstPath.txt and DMU Installations					 #
##############################################################################

use strict;
use File::Glob;
use File::Path;
use File::Find;
use File::Copy;

my @robocopy;
my @source;
my @destination;
my @replace_source;
my @replace_destination;
my @env_files;
my @env_files_old;
my $i;
my $replication_default_mode="all";
my $replication_logfile;
my $replication_logstore;
my $replication_custom_mode="none";
my $replication_mode;
my $is_empty;

if ( -e "$ENV{CABUM_REPCONF}" ) {
	&readconfig($ENV{CABUM_REPCONF});
} else {
	print "Config-file not found!\n";
}

open(LOG, ">:encoding(UTF-8)", "$replication_logfile");

&logging("INFO", "sctrep.pl start logging...");

if (-e "$replication_logstore") {
	&logging("INFO", "Logstore \"$replication_logstore\" exists...");
} else {
	&logging("INFO", "Logstore \"$replication_logstore\" does not exist...trying to create...");
	mkdir("$replication_logstore");
	if (-e "$replication_logstore") {
		&logging("INFO", "Logstore \"$replication_logstore\" was created...");
    } else {
		&logging("INFO", "Logstore \"$replication_logstore\" could not be created...");
	}
}


my $path=$ENV{CABUM_WORK};
$path =~ s/\\/\//g;
foreach (glob("$path/sctrep_*.log")) {
	&logging("INFO", "deleting $_...");
    unlink "$_";
}

&logging("INFO", "default mode: $replication_default_mode");
if ($replication_custom_mode !~ m/none/ ) {&logging("INFO", "custom mode: $replication_custom_mode")};
&logging("INFO", "robocopy error logs: $replication_logstore");

&setrobocopy;
for ($i=1;$i<@robocopy;$i++) {
	&logging("INFO", "robocopy option " . $i . ": $robocopy[$i]");
}

if ( -e "$ENV{CATMY_REF}\\cabum\\CatInstPath.txt" ) {
	&logging("INFO", "reading \"$ENV{CATMY_REF}\\cabum\\CatInstPath.txt\"...");
	&readconfig_path("$ENV{CATMY_REF}\\cabum\\CatInstPath.txt");
} else {
	print "CatInstPath.txt not found!\n";
	&logging("INFO", "\"$ENV{CATMY_REF}\\cabum\\CatInstPath.txt\" not found");
}

if ( -e "$ENV{CATMY_REF}\\cabum\\NXInstPath.txt" ) {
	&logging("INFO", "reading \"$ENV{CATMY_REF}\\cabum\\NXInstPath.txt\"...");
	&readconfig_path("$ENV{CATMY_REF}\\cabum\\NXInstPath.txt");
} else {
	print "NXInstPath.txt not found!\n";
	&logging("INFO", "\"$ENV{CATMY_REF}\\cabum\\NXInstPath.txt\" not found");
}

if ( $replication_custom_mode !~ m/none/ ) { 
	$replication_mode = $replication_custom_mode;
} else {
	$replication_mode = $replication_default_mode;
}

if ( $replication_mode =~ m/all/ ) {
	print "\n*** Replication mode: all ***\n\n";
	push(@source,$ENV{CATPRO_REF});
	push(@destination,$ENV{CATPRO_LOC});
	push(@source,$ENV{CATSCT_REF_CATIA});
	push(@destination,$ENV{CATSCT_LOC_CATIA});
	push(@source,$ENV{CATSCT_REF_NX});
	push(@destination,$ENV{CATSCT_LOC_NX});
	push(@source,$ENV{CATMY_REF});
	push(@destination,$ENV{CATMY_LOC});
	push(@source,$ENV{CATCUS_REF});
	push(@destination,$ENV{CATCUS_LOC});
	push(@source,$ENV{FDEC_REF});
	push(@destination,$ENV{FDEC_LOC});
	&start_robocopy;
}

if ( $replication_mode =~ m/scan/) {
	print "\n*** Replication mode: scan ***\n\n";
	my $site_sct;
	my $site_my;
	my $exe_path;
	my $cad_type;
	my @replicate;
	my @delete;
	my $found;
	&collect_env_files;
	&collect_env_files_old;
	foreach (@env_files) {
		&logging("INFO", "scanning \"$_\"");
		$found=0;
		($cad_type, $exe_path, $site_sct, $site_my) = &source_env($_);
		if ($cad_type eq "catia") {
			if (-e "$exe_path/code/bin/catstart.exe") {
				$found=1;
				&logging("INFO", "\"$exe_path/code/bin/catstart.exe\" found");
			} else {
				&logging("WARNING", "\"$exe_path/code/bin/catstart.exe\" not found");
				for ($i=0;$i<@replace_source;$i++) {
					my $temp=$exe_path;
					$temp=~s/$replace_source[$i]/$replace_destination[$i]/g;
					if (-e "$temp/code/bin/catstart.exe") {
						$found=1;
						&logging("INFO", "\"$temp/code/bin/catstart.exe\" (substituted) found");
					}
				}
			}
		}
		if ($cad_type eq "nx") {
			if (-e "$exe_path/ugraf.exe") {
				$found=1;
				&logging("INFO", "\"$exe_path/ugraf.exe\" found");
			} else {
				&logging("WARNING", "\"$exe_path/ugraf.exe\" not found");
				for ($i=0;$i<@replace_source;$i++) {
					my $temp=$exe_path;
					$temp=~s/$replace_source[$i]/$replace_destination[$i]/g;
					if (-e "$temp/ugraf.exe") {
						$found=1;
						&logging("INFO", "\"$temp/ugraf.exe\" (substituted) found");
					}
				}
			}
		}
		if ($found==0) {
			if ($site_sct) {push(@delete, $site_sct)};
			if ($site_my) {push(@delete, $site_my)};
		}
		if ($found==1) {
			if ($site_sct) {push(@replicate, $site_sct)};
			if ($site_my) {push(@replicate, $site_my)};
			$found=0;
		}
	}
	foreach (@env_files_old) {
		&logging("INFO", "scanning \"$_\"");
		$found=0;
		my $env=$_;
		my $temp1=$ENV{CATREP_REF};
		$temp1 =~ s/\\/\//g;
		my $temp2=$ENV{CATREP_LOC};
		$temp2 =~ s/\\/\//g;
		$env =~ s/\\/\//g;
		$env =~ s/$temp1/$temp2/g;
		$env =~ s/\//\\/g;
		($cad_type, $exe_path, $site_sct, $site_my) = &source_env($_);
		if ($cad_type eq "catia") {
			if (-e "$exe_path\\code\\bin\\catstart.exe") {
				$found=1;
				&logging("INFO", "\"$exe_path\\code\\bin\\catstart.exe\" found");
			} else {
				&logging("WARNING", "\"$exe_path\\code\\bin\\catstart.exe\" not found");
				for ($i=0;$i<@replace_source;$i++) {
					my $temp=$exe_path;
					$temp=~s/$replace_source[$i]/$replace_destination[$i]/g;
					if (-e "$temp\\code\\bin\\catstart.exe") {
						$found=1;
						&logging("INFO", "\"$temp\\code\\bin\\catstart.exe\" (substituted) found");
					}
				}
			}
		}
		if ($found==0) {
			if ($env) {push(@delete, $env)};
			if ($site_sct) {push(@delete, $site_sct)};
			if ($site_my) {push(@delete, $site_my)};
		}
		if ($found==1) {
			if ($env) {push(@replicate, $env)};
			if ($site_sct) {push(@replicate, $site_sct)};
			if ($site_my) {push(@replicate, $site_my)};
			$found=0;
		}
	}

	my @path=($ENV{CATSCT_LOC_CATIA},$ENV{CATSCT_LOC_NX},$ENV{CATMY_LOC_CATIA},$ENV{CATMY_LOC_NX}); 
	foreach (@path) {
		my $loc_path=$_;
		my $ref_path=$_;
		$ref_path =~ s/\Q$ENV{CATREP_LOC_3DX}/$ENV{CATREP_REF}/g;
		$loc_path =~ s/\\/\//g;
		$ref_path =~ s/\\/\//g;
		foreach (glob("$loc_path/*")) {
			my @temp_path=split(/\//,$_);
			if (!-e "$ref_path/$temp_path[-1]") {
				push(@delete,$_);
				$delete[-1] =~ s/\//\\/g;
				&logging("WARNING", "\"$delete[-1]\" does not exist on server...will be deleted");
			}
			if (-e "$_" && -d "$_") {
				foreach (glob("$_/*")) {
					my @temp_path2=split(/\//,$_);
					if (!-e "$ref_path/$temp_path2[-2]/$temp_path2[-1]") {
						push(@delete,$_);
						$delete[-1] =~ s/\//\\/g;
						&logging("WARNING", "\"$delete[-1]\" does not exist on server...will be deleted");
					}
				}
			}
		}
	}

	splice(@replicate,0,@replicate,&remove_duplicates(@replicate));
	splice(@delete,0,@delete,&remove_duplicates(@delete));
	splice(@delete,0,@delete,&filter_delete(\@replicate,\@delete));
	
	push(@source,$ENV{CATPRO_REF});
	push(@destination,$ENV{CATPRO_LOC});
	push(@source,$ENV{CATMY_REF} . "\\cabum");
	push(@destination,$ENV{CATMY_LOC} . "\\cabum");
	push(@source,$ENV{CATMY_REF} . "\\othercad");
	push(@destination,$ENV{CATMY_LOC} . "\\othercad");
	push(@source,$ENV{CATMY_REF} . "\\catia\\all");
	push(@destination,$ENV{CATMY_LOC} . "\\catia\\all");
	push(@source,$ENV{CATMY_REF} . "\\nx\\all");
	push(@destination,$ENV{CATMY_LOC} . "\\nx\\all");
	push(@source,$ENV{CATCUS_REF} . "\\env");
	push(@destination,$ENV{CATCUS_LOC} . "\\env");
	push(@source,$ENV{FDEC_REF} . "\\env");
	push(@destination,$ENV{FDEC_LOC} . "\\env");
	push(@source,$ENV{FDEC_REF} . "\\site\\all");
	push(@destination,$ENV{FDEC_LOC} . "\\site\\all");
	foreach (@replicate) {
		push(@destination, $_);
		my $temp1=$ENV{CATSCT_LOC};
		$temp1 =~ s/\\/\//g;
		my $temp2=$ENV{CATSCT_REF};
		$temp2 =~ s/\\/\//g;
		my $temp3=$ENV{CATMY_LOC};
		$temp3 =~ s/\\/\//g;
		my $temp4=$ENV{CATMY_REF};
		$temp4 =~ s/\\/\//g;
		my $temp5=$ENV{CATCUS_LOC};
		$temp5 =~ s/\\/\//g;
		my $temp6=$ENV{CATCUS_REF};
		$temp6 =~ s/\\/\//g;
		my $temp7=$ENV{FDEC_LOC};
		$temp7 =~ s/\\/\//g;
		my $temp8=$ENV{FDEC_REF};
		$temp8 =~ s/\\/\//g;
		$_ =~ s/\\/\//g;
		$_ =~ s/$temp1/$temp2/g;
		$_ =~ s/$temp3/$temp4/g;
		$_ =~ s/$temp5/$temp6/g;
		$_ =~ s/$temp7/$temp8/g;
		$_ =~ s/\//\\/g;
		push(@source, $_);
	}
	&start_robocopy;
	&start_delete(@delete);
}

&logging("INFO", "...end logging");

close(LOG);

# Subprograms

sub logging {
	my $temp1=$_[0];
	my $temp2=$_[1];
	chomp $temp1;
	chomp $temp2;
	print LOG localtime() . " - $temp1: $temp2\n";
}

sub start_robocopy {
	my $errorlevel;
	my $error_copy;
	my @copy;
	$copy[0]="copy";
	$copy[3]="/y";
	$copy[4]="1>nul";
	$copy[5]="2>&1";
	for ($i=0;$i<@source;$i++) {
		if ($source[$i]=~m/\.\w+$/) {
			if (-e $source[$i]) {
				$copy[1]="$source[$i]";
				$copy[2]="$destination[$i]";
				print "Copy " . $source[$i] . "\n";
				&logging("INFO", "running: @copy");
				system("cmd", "/c", @copy);
			}
		} else {
			if (-d $source[$i]) {
				my @robocopy_command;
				@robocopy_command=@robocopy;
				my (@temp1) = split(/\\/,$source[$i]);
				my $log_temp=$source[$i];
				$log_temp =~ s/\Q$ENV{CATREP_REF}//g;
				$log_temp =~ s/\\/_/g;
				push (@robocopy_command,"/log:\"$ENV{CABUM_WORK}\\sctrep$log_temp.log\"");
				push (@robocopy_command,"$source[$i]");
				push (@robocopy_command,"$destination[$i]");
				push (@robocopy_command,"1>nul");
				push (@robocopy_command,"2>&1");
				print "Replicate $source[$i]...";
				my $robocopy_command=join(" ",@robocopy_command);
				&logging("INFO", "Replicate $source[$i]...");
				&logging("INFO", "running: $robocopy_command");
				system($robocopy_command);
				$errorlevel=$? >> 8;
				if ($errorlevel<8) {
					print "success (return code: $errorlevel)\n";
					&logging("INFO", "...success (return code: $errorlevel)");
					if (-e "$replication_logstore\\$ENV{COMPUTERNAME}_$ENV{USERNAME}_sctrep$log_temp.log") {
						unlink "$replication_logstore\\$ENV{COMPUTERNAME}_$ENV{USERNAME}_sctrep$log_temp.log";
					}
				}
				if ($errorlevel>7) {
					print "error (return code: $errorlevel)\n";
					&logging("ERROR", "...error (return code: $errorlevel)");
					my $copy_command="copy \"$ENV{CABUM_WORK}\\sctrep$log_temp.log\" \"$replication_logstore\\$ENV{COMPUTERNAME}_$ENV{USERNAME}_sctrep$log_temp.log\" 1>nul 2>&1";
					$copy_command =~ s/\//\\/g;
					system($copy_command);
					$error_copy=$? >> 8;
					if ($error_copy == 0) {
                        print "=> Logfile copied to: \"$replication_logstore/$ENV{COMPUTERNAME}_$ENV{USERNAME}_sctrep$log_temp.log\"\n";
						&logging("INFO", "Logfile copied to: \"$replication_logstore/$ENV{COMPUTERNAME}_$ENV{USERNAME}_sctrep$log_temp.log\"");
                    } else {
						print "=> !!!Logfile could not be copied!!!\n";
						&logging("ERROR", "!!!Logfile could not be copied!!!");
					}
                    
				}
			}
		}
	}
}

sub start_delete {
	my @rd;
	$rd[0]="rd";
	$rd[2]="/s";
	$rd[3]="/q";
	$rd[4]="1>nul";
	$rd[5]="2>&1";
	foreach (@_) {
		if ($_=~m/\.\w+$/) {
			if (-e $_ && -f $_) {
				print "Deleting " . $_ . "\n";
				&logging("INFO", "deleting: $_");
				unlink "$_";
			}
		} else {
			if (-e $_ && -d $_) {
				$rd[1]="$_";
				print "Deleting " . $_ . "\n";
				&logging("INFO", "deleting: $_");
				system("cmd", "/c", @rd);
			}
		}
		my $deleted=1;
		while ($deleted==1) {
			$_ = substr($_,0,rindex($_,"\\"));
			$is_empty=1;
			if (uc($_) eq uc($ENV{CATREP_LOC_3DX})) {$is_empty=0};
			$deleted=0;
			eval {find(\&check_dir, $_)};
			if ($is_empty==1) {
				if (-e $_ && -d $_ && not $_ =~ m/^\w:$/) {
					print "Deleting " . $_ . "\n";
					$rd[1]="$_";
					&logging("INFO", "deleting: $_");
					system("cmd", "/c", @rd);
                }
				$deleted=1
			}
		}
	}
}

sub check_dir {
	if ($_ ne ".") {
		$is_empty=0;
		die;
	}
}

sub remove_duplicates {
	my @unique = ();
	my %seen   = ();

	foreach (@_) {
		next if $seen{$_}++;
		push @unique, $_;
	}
	return @unique;
}

sub filter_delete {
	my $replicate=shift;
	my $delete=shift;
	my %hashtemp;
	
	foreach (@$replicate) {$hashtemp{$_}++}
	
	for ($i=0;$i<@$delete;$i++) {
		if ($hashtemp{@$delete[$i]}) {
			splice(@$delete,$i,1);
		}
		
	}
	return @$delete;
}

sub collect_env_files {
	my $path=$ENV{CATSCT_REF};
	$path =~ s/\\/\//g;
	foreach (glob("$path/*/*/*/env/*.*")) {
		if ( substr($_,-3) ne "xml" && substr($_,-6) ne "sample" && substr($_,-4) ne "user" ) { push (@env_files, $_) };
	}
	$path=$ENV{CATMY_REF};
	$path =~ s/\\/\//g;
	foreach (glob("$path/*/*/*/env/*.*")) {
		if ( substr($_,-3) ne "xml" && substr($_,-6) ne "sample" && substr($_,-4) ne "user" ) { push (@env_files, $_) };
	}
}

sub collect_env_files_old {
	my $path=$ENV{CATCUS_REF};
	$path =~ s/\\/\//g;
	foreach (glob("$path/env/*.*")) {
		if ( substr($_,-3) ne "xml" && substr($_,-6) ne "sample" && substr($_,-4) ne "user" ) { push (@env_files_old, $_) };
	}
	$path=$ENV{FDEC_REF};
	$path =~ s/\\/\//g;
	foreach (glob("$path/env/*.*")) {
		if ( substr($_,-3) ne "xml" && substr($_,-6) ne "sample" && substr($_,-4) ne "user" ) { push (@env_files_old, $_) };
	}
}

sub source_env {
	open(FILE, $_[0]);
	my @env=<FILE>;
	close(FILE);
	my %environment;
	my %environment_filtered;
	my @env_filtered;
	my $left;
	my $right;
	for (keys %ENV) {
		$environment{$_} = $ENV{$_};
	}
	for ($i=0;$i<@env;$i++) {
		if ( ($env[$i] !~ m/^!+/) && ($env[$i] !~ /^\s*$/) ) {
			chomp($env[$i]);
			if ( not rindex($env[$i],"=") == -1 ) {
				push(@env_filtered,$env[$i]);
				$environment{substr($env[$i],0,rindex($env[$i],"="))}=substr($env[$i],rindex($env[$i],"=")+1,length($env[$i])-(rindex($env[$i],"=")+1));
			}
		}
	}
	for ($i=0;$i<@env_filtered;$i++) {
		($left, $right)=split("=",$env_filtered[$i]);
		$right =~ s/\%([^\%]+)\%/$environment{$1}/g;
		$env_filtered[$i]=$left . "=" . $right;
		$environment{$left}=$right;
		$environment_filtered{$left}=$right;
	}
	my $site_sct;
	my $site_my;
	my $exe_path;
	my $cad_type;
	if (exists($environment_filtered{"CAT_SITE"})) { $site_sct=$environment_filtered{"CAT_SITE"} };
	if (exists($environment_filtered{"NX_SITE"})) { $site_sct=$environment_filtered{"NX_SITE"} };
	if (exists($environment_filtered{"CAT_SITE_MY"})) { $site_my=$environment_filtered{"CAT_SITE_MY"} };
	if (exists($environment_filtered{"CAT_SITE_FDEC"})) { $site_my=$environment_filtered{"CAT_SITE_FDEC"} };
	if (exists($environment_filtered{"NX_SITE_MY"})) { $site_my=$environment_filtered{"NX_SITE_MY"} };
	if (exists($environment_filtered{"CATInstallPath"})) {
		$exe_path=$environment_filtered{"CATInstallPath"};
		($exe_path)=split(";",$exe_path);
		$exe_path=~s/\\/\//g;
		$exe_path=lc($exe_path);
		$cad_type="catia";
	}
	if (exists($environment_filtered{"UGII_ROOT_DIR"})) {
		$exe_path=$environment_filtered{"UGII_ROOT_DIR"};
		$exe_path=~s/\\/\//g;
		$exe_path=lc($exe_path);
		$cad_type="nx";
	}
	
	return($cad_type, $exe_path, $site_sct, $site_my);
}

sub readconfig {
	&logging("INFO", "reading config (\"$_[0]\")...");
	open(FILE,"$_[0]");
	my @config=<FILE>;
	close(FILE);
	my %environment;
	for (keys %ENV) {
		$environment{$_} = $ENV{$_};
	}
	for ($i=0;$i<@config;$i++) {
		if ( ($config[$i] !~ m/^#+/) && ($config[$i] !~ /^\s*$/) ) { 
			if ( not rindex($config[$i],"=") == -1 ) {
				$config[$i] =~ s/^\s+//;
				$config[$i] =~ s/\s+$//;
				if ( substr($config[$i],0,rindex($config[$i],"=")) =~ m/default/i ) {
					$replication_default_mode = substr($config[$i],rindex($config[$i],"=")+1,length($config[$i])-(rindex($config[$i],"=")+1));
				}
				if ( substr($config[$i],0,rindex($config[$i],"=")) =~ m/logfile/i ) {
					$replication_logfile = substr($config[$i],rindex($config[$i],"=")+1,length($config[$i])-(rindex($config[$i],"=")+1));
					$replication_logfile =~ s/\!([^\!]+)\!/$environment{$1}/g;
					$replication_logfile =~ s/\\/\//g;
				}
				if ( substr($config[$i],0,rindex($config[$i],"=")) =~ m/logstore/i ) {
					$replication_logstore = substr($config[$i],rindex($config[$i],"=")+1,length($config[$i])-(rindex($config[$i],"=")+1));
					$replication_logstore =~ s/\!([^\!]+)\!/$environment{$1}/g;
					$replication_logstore =~ s/\\/\//g;
				}
				if ( substr($config[$i],0,rindex($config[$i],"=")) =~ m/$ENV{COMPUTERNAME}/i ) {
					$replication_custom_mode = substr($config[$i],rindex($config[$i],"=")+1,length($config[$i])-(rindex($config[$i],"=")+1));
				}
			}
		}
	}
	if ($replication_logfile eq '') {
        $replication_logfile="$ENV{CABUM_WORK}/sctrep.log";
    }
	if ($replication_logstore eq '') {
        $replication_logstore="$ENV{CATREP_REF}/../log";
    }
}

sub readconfig_path {
	open(FILE,"$_[0]");
	my @config=<FILE>;
	close(FILE);
	my @split_result;
	foreach (@config) {
		if ( ($_ !~ m/^#+/) && ($_ !~ /^\s*$/) ) {
			chomp $_;
			my $count = $_ =~ tr/"//;
			if ($count == 4) {
				chop $_; 
				$_ = substr ($_,1); 
				$_ =~ s/"\s"/;/g;
			}
			elsif ($count == 2) {
				if ($_ !~ m/^"+/) {
					chop $_; 
					$_ =~ s/\s"/;/g;
				} else {
					$_ = substr ($_,1); 
					$_ =~ s/"\s/;/g;
				}			
			} else {
				$_ =~ s/\s/;/g;
			}
		@split_result = split ( ';', lc($_) );
		&logging("INFO", "adding path replacement \"$split_result[0]\" -> \"$split_result[1]\"");
		push (@replace_source, $split_result[0]);
		push (@replace_destination, $split_result[1]);
		$replace_source[-1] =~ s/\\/\//g;
		$replace_destination[-1] =~ s/\\/\//g;
		}
	}
}			


sub setrobocopy {
	if ( defined $ENV{osversion} && $ENV{osversion} >= 7 ) {
		if ( -e "$ENV{SystemRoot}\\system32\\robocopy.exe" ) { push(@robocopy, "$ENV{SystemRoot}\\system32\\robocopy.exe") };
		push(@robocopy,"/mir");
		for (keys %ENV) {
			if ( $_ =~ m/^robocopy_/i ) {
				if ($ENV{$_} =~ m/true/i) {
                    push(@robocopy, "/$_");
					$robocopy[-1] =~ s/robocopy_//i;
                }
				if ( ($ENV{$_} !~ m/true/i) && ($ENV{$_} !~ m/false/i) ) {
                    push(@robocopy, "/$_:$ENV{$_}");
					$robocopy[-1] =~ s/robocopy_//i;
                }
            }
		}
	} else {
		if ( -e "$ENV{CATPRO_REF}\\bin\\robocopy.exe" ) { push(@robocopy, "$ENV{CATPRO_REF}\\bin\\robocopy.exe") };
		if ( -e "$ENV{CATPRO_LOC}\\bin\\robocopy.exe" ) { push(@robocopy, "$ENV{CATPRO_LOC}\\bin\\robocopy.exe") };
		push(@robocopy,"/mir");
		for (keys %ENV) {
			if ( $_ eq "robocopy_r" ) {
				if ( ($ENV{$_} !~ m/true/i) || ($ENV{$_} !~ m/false/i) ) {
                    push(@robocopy, "/$_:$ENV{$_}");
                }
            }
			if ( $_ eq "robocopy_w" ) {
				if ( ($ENV{$_} !~ m/true/i) || ($ENV{$_} !~ m/false/i) ) {
                    push(@robocopy, "/$_:$ENV{$_}");
                }
            }
		}
	}
	if ( not defined $robocopy[0] ) {
		print "robocopy.exe not found!";
		exit 1;
	}
}
